=== modified file 'NEWS'
--- NEWS	2008-09-01 23:49:18 +0000
+++ NEWS	2008-09-02 05:28:37 +0000
@@ -54,6 +54,9 @@
     * Merging from a previously joined branch will no longer cause 
       a traceback. (Jelmer Vernooij, #203376)
 
+    * Pack operations on windows network shares will work even with large
+      files. (Robert Collins, #255656)
+
     * Running ``bzr st PATH_TO_TREE`` will no longer suppress merge
       status. Status is also about 7% faster on mozilla sized trees
       when the path to the root of the tree has been given. Users of

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2008-08-31 19:06:07 +0000
+++ bzrlib/osutils.py	2008-09-02 05:28:37 +0000
@@ -569,6 +569,24 @@
     return length
 
 
+def pump_string_file(bytes, file_handle, segment_size=None):
+    """Write bytes to file_handle in many smaller writes.
+
+    :param bytes: The string to write.
+    :param file_handle: The file to write to.
+    """
+    # Write data in chunks rather than all at once, because very large
+    # writes fail on some platforms (e.g. Windows with SMB  mounted
+    # drives).
+    if not segment_size:
+        segment_size = 5242880 # 5MB
+    segments = range(len(bytes) / segment_size + 1)
+    write = file_handle.write
+    for segment_index in segments:
+        segment = buffer(bytes, segment_index * segment_size, segment_size)
+        write(segment)
+
+
 def file_iterator(input_file, readsize=32768):
     while True:
         b = input_file.read(readsize)

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2008-07-17 20:16:43 +0000
+++ bzrlib/tests/test_osutils.py	2008-08-15 05:35:17 +0000
@@ -37,6 +37,7 @@
         is_inside_or_parent_of_any,
         pathjoin,
         pumpfile,
+        pump_string_file,
         )
 from bzrlib.tests import (
         probe_unicode_in_user_encoding,
@@ -440,6 +441,30 @@
             message = "Data not equal.  Expected %d bytes, received %d."
             self.fail(message % (len(response_data), self.test_data_len))
 
+
+class TestPumpStringFile(TestCase):
+
+    def test_empty(self):
+        output = StringIO()
+        pump_string_file("", output)
+        self.assertEqual("", output.getvalue())
+
+    def test_more_than_segment_size(self):
+        output = StringIO()
+        pump_string_file("123456789", output, 2)
+        self.assertEqual("123456789", output.getvalue())
+
+    def test_segment_size(self):
+        output = StringIO()
+        pump_string_file("12", output, 2)
+        self.assertEqual("12", output.getvalue())
+
+    def test_segment_size_multiple(self):
+        output = StringIO()
+        pump_string_file("1234", output, 2)
+        self.assertEqual("1234", output.getvalue())
+
+
 class TestSafeUnicode(TestCase):
 
     def test_from_ascii_string(self):

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2008-08-28 15:31:34 +0000
+++ bzrlib/transport/__init__.py	2008-09-02 05:28:37 +0000
@@ -254,7 +254,7 @@
         self.file_handle.close()
 
     def write(self, bytes):
-        self.file_handle.write(bytes)
+        osutils.pump_string_file(bytes, self.file_handle)
 
 
 class AppendBasedFileStream(FileStream):

